# -*- coding: utf-8 -*-
"""
Marches aléatoires
TP conçu par Stéphane Gonnord pour PSI999, 2019/2020
Usage : python3 marches_alea.py
"""

__author__ = "Stéphane Gonnord, modifié à la marge par Laure Gonnord <Laure.Gonnord@univ-lyon1.fr>"
__version__ = "0.4 Juin 2020"


from numpy.random import random, randint
import matplotlib.pyplot as pypl
from numpy import sqrt, array, cos, sin, pi


"""
MARCHES DANS UNE RUE, avec ou sans biais
"""

def pas(p):  # p = proba d'aller à droite
    if random() < p:
        return 1
    else:
        return -1


def marche(nb_pas, p):
    X, m = 0, [0]
    # TODO : Une marche à p pas.
    # BEGIN CUT
    for _ in range(nb_pas):
        X += pas(p)
        m.append(X)
    # END CUT
    return m

def dessine_une_marche_aux(nb_pas, p):
    les_t = range(nb_pas+1)
    les_y = marche(nb_pas, p)
    if not len(les_y) == nb_pas+1 :
        print("erreur: la marche que vous me demandez de dessiner ne fait pas la bonne taille")
        exit(0)
    else:
        pypl.plot(les_t, les_y)

def dessine_marches():
    for nb_pas in [100, 1000, 10000]:
        pypl.clf()
        for _ in range(10):
            dessine_une_marche_aux(nb_pas, 0.5)
            pypl.grid()
        pypl.savefig('marche_%i.pdf' % nb_pas) # attention à l'indentation ici.
        pypl.clf()
        for _ in range(10):
            dessine_une_marche_aux(nb_pas, 0.7)
        pypl.grid()
        pypl.savefig('marche_biaisee_%i.pdf' % nb_pas)


def valeur_moyenne(nb_pas, p, nb_marches):
    res = 0
    # TODO 
    # BEGIN CUT
    res = sum(marche(nb_pas, p)[-1] for _ in range(nb_marches)) / nb_marches
    # END CUT
    return res



def distance_moyenne(nb_pas, p, nb_marches):
    res = 0
    # TODO
    # BEGIN CUT
    res = sum(abs(marche(nb_pas, p)[-1]) for _ in range(nb_marches)) / nb_marches
    # END CUT
    return res 
    


def distance_carre_moyenne(nb_pas, p, nb_marches):
    # TODO
    # BEGIN CUT
    return sum(marche(nb_pas, p)[-1]**2 for _ in range(nb_marches)) / nb_marches
    # END CUT
    # UNCOMMENT return 0



def repasse_par(objectif, nb_pas, p):  # toujours avec p = proba(droite)
    x = 0
    # TODO
    # BEGIN CUT
    for _ in range(nb_pas):
        x += pas(p)
        if x == objectif:
            return True
    # END CUT
    return False


def proba_passage(objectif, nb_pas, nb_marches, p):
    cpt = 0
    # TODO
    # BEGIN CUT
    for _ in range(nb_marches):
        if repasse_par(objectif, nb_pas, p):
            cpt += 1
    # END CUT
    return cpt/nb_marches

if __name__ == '__main__':
    # LG : chez moi lancer cela prend 4 secondes utilisateur et cela génère 6 fichiers:
    # marche_biaisee_10000.pdf  marches_alea.py marche_10000.pdf  marche_biaisee_1000.pdf 
    # marche_1000.pdf   marche_biaisee_100.pdf   marche_100.pdf

    print(" 3 marches dans Z" )
    for i in range(3):
        print(marche(10, 0.5))
    print(" 1 marche biaisée dans Z" )
    print(marche(10, 0.3))
    print("dessin de marches regarder les fichiers générés") 
    dessine_marches()
    # à vous, tester les probas, les valeurs moyennes, etc
    print("tests persos")
    # TODO
    # BEGIN CUT
    for i in range(3):
        print(valeur_moyenne(100, 0.5, 10))
    print(valeur_moyenne(100, 0.5, 10**4))
    print(proba_passage(0, 3, 10**4, 0.5))
    print(distance_carre_moyenne(100, 0.5, 10**4))
    
"""
>>> valeur_moyenne(100, 0.5, 10)
1.0
>>> valeur_moyenne(100, 0.5, 10)
-5.4
>>> valeur_moyenne(100, 0.5, 10**4)
0.1628

>>> valeur_moyenne(100, 0.7, 10**3)
39.87
>>> valeur_moyenne(100, 0.2, 10**3)
-59.858
"""

"""
>>> proba_passage(0, 2, 10**4, 0.5)
0.4978
>>> proba_passage(0, 3, 10**4, 0.5)
0.5021
>>> proba_passage(0, 4, 10**5, 0.5)
0.62563
>>> 5/8
0.625

>>> proba_passage(0, 10, 10**5, 0.5)
0.7553
>>> proba_passage(0, 100, 10**5, 0.5)
0.91979
>>> proba_passage(0, 10**4, 10**4, 0.5)
0.9913

>>> proba_passage(-5, 4, 10**5, 0.5)
0.0
>>> proba_passage(-5, 10, 10**5, 0.5)
0.10912
>>> proba_passage(-5, 100, 10**5, 0.5)
0.61873
>>> proba_passage(-5, 10**4, 10**4, 0.5)
0.9637

>>> proba_passage(-5, 10, 10**5, 0.8)
0.00071
>>> proba_passage(-5, 10, 10**5, 0.2)
0.75681
>>> proba_passage(-5, 100, 10**5, 0.8)
0.00107
>>> proba_passage(-5, 100, 10**5, 0.2)
1.0
>>> proba_passage(-5, 10**4, 10**3, 0.8)
0.001
>>> proba_passage(-5, 10**4, 10**3, 0.2)
1.0

>>> distance_moyenne(10, 0.5, 10**4)
2.4818
>>> distance_carre_moyenne(10, 0.5, 10**4)
10.1388

>>> distance_moyenne(100, 0.5, 10**4)
7.9222
>>> distance_carre_moyenne(100, 0.5, 10**4)
99.1316

"""        
    # END CUT
