# -*- coding: utf-8 -*-
"""
Marches aléatoires
TP conçu par Stéphane Gonnord pour PSI999, 2019/2020
Usage : python3 marches_alea.py
"""

__author__ = "Stéphane Gonnord, modifié à la marge par Laure Gonnord <Laure.Gonnord@univ-lyon1.fr>"
__version__ = "0.4 Juin 2020"


from numpy.random import random, randint
import matplotlib.pyplot as pypl
from numpy import sqrt, array, cos, sin, pi


"""
  DANS LE PLAN puis l'espace
"""


def pas2():  # 4 directions équiprobables
    return [[1, 0], [0, 1], [-1, 0], [0, -1]][randint(0, 4)]


def marche2(nb_pas):
    X, Y, mX, mY = 0, 0, [0], [0]
    # TODO
    # BEGIN CUT
    for _ in range(nb_pas):
        dx, dy = pas2()
        X += dx
        Y += dy
        mX.append(X)
        mY.append(Y)
    # END CUT
    return mX, mY


def dessine_une_marche2_aux(nb_pas):
    les_x, les_y = marche2(nb_pas)
    pypl.clf()
    pypl.plot(les_x, les_y)
    pypl.plot([0, les_x[-1]], [0, les_y[-1]], marker='o', color='red')
    pypl.grid()
    pypl.savefig('marche_plan_%i.pdf' % nb_pas)


def dessine_marche2():
    for nb_pas in [100, 10000]:
        dessine_une_marche2_aux(nb_pas)

def distance_moyenne2(nb_pas, nb_marches):
    s = 0
    # TODO
    # BEGIN CUT
    for _ in range(nb_marches):
        mx, my = marche2(nb_pas)
        x, y = mx[-1], my[-1]
        s += sqrt(x**2+y**2)
    # END CUT
    return s / nb_marches

def distance_carre_moyenne2(nb_pas, nb_marches):
    s = 0
    # TODO
    # BEGIN CUT
    for _ in range(nb_marches):
        mx, my = marche2(nb_pas)
        x, y = mx[-1], my[-1]
        s += x**2+y**2
    # END CUT
    return s / nb_marches

def repasse_par2(objectif, nb_pas):
    x, y = 0, 0
    # TODO
    # BEGIN CUT
    for _ in range(nb_pas):
        dx, dy = pas2()
        x += dx
        y += dy
        if (x, y) == objectif:
            return True
    # END CUT
    return False


def proba_passage2(objectif, nb_pas, nb_marches):
    cpt = 0
    # TODO
    # BEGIN CUT
    for _ in range(nb_marches):
        if repasse_par2(objectif, nb_pas):
            cpt += 1
    # END CUT
    return cpt/nb_marches


"""
  DANS L'ESPACE
"""

# TODO à vous !

# BEGIN CUT


def pas3():  # 6 directions équiprobables
    return [[1, 0, 0], [-1, 0, 0], [0, 1, 0], [0, -1, 0],
            [0, 0, 1], [0, 0, -1]][randint(0, 6)]


def marche3(nb_pas):
    X, Y, Z, mX, mY, mZ = 0, 0, 0, [0], [0], [0]
    for _ in range(nb_pas):
        dx, dy, dz = pas3()
        X += dx
        Y += dy
        Z += dz
        mX.append(X)
        mY.append(Y)
        mZ.append(Z)
    return mX, mY, mZ


def distance_moyenne3(nb_pas, nb_marches):
    s = 0
    for _ in range(nb_marches):
        mx, my, mz = marche3(nb_pas)
        x, y, z = mx[-1], my[-1], mz[-1]
        s += sqrt(x**2+y**2+z**2)
    return s / nb_marches


def distance_carre_moyenne3(nb_pas, nb_marches):
    s = 0
    for _ in range(nb_marches):
        mx, my, mz = marche3(nb_pas)
        x, y, z = mx[-1], my[-1], mz[-1]
        s += x**2+y**2+z**2
    return s / nb_marches

"""
>>> distance_moyenne3(10, 10**4)
2.9089227885883364
>>> distance_carre_moyenne3(10, 10**4)
9.9872

>>> distance_moyenne3(100, 10**4)
9.1756848430914975
>>> distance_carre_moyenne3(100, 10**4)
98.3932
"""


def repasse_par3(objectif, nb_pas):
    x, y, z = 0, 0, 0
    for _ in range(nb_pas):
        dx, dy, dz = pas3()
        x += dx
        y += dy
        z += dz
        if (x, y, z) == objectif:
            return True
    return False


def proba_passage3(objectif, nb_pas, nb_marches):
    cpt = 0
    for _ in range(nb_marches):
        if repasse_par3(objectif, nb_pas):
            cpt += 1
    return cpt/nb_marches

"""
>>> proba_passage3((0, 0, 0), 2, 10**4)
0.1651

>>> proba_passage3((0, 0, 0), 10, 10**4)
0.251
>>> proba_passage3((0, 0, 0), 10, 10**6)
0.252899

>>> proba_passage3((0, 0, 0), 100, 10**4)
0.3134
>>> proba_passage3((0, 0, 0), 1000, 10**4)
0.3309
>>> proba_passage3((0, 0, 0), 10**4, 10**2)
0.34
>>> proba_passage3((0, 0, 0), 10**5, 10**2)
0.35
"""
# END CUT

"""
  SUR LE CERCLE TRIGONOMÉTRIQUE 
"""

# TODO à vous !

# BEGIN CUT
def marche_trigo(nb_pas, alpha):
    marche_angles = alpha * array(marche(nb_pas, 0.5))
    mx, my = cos(marche_angles), sin(marche_angles)
    return mx, my


"""
>>> marche_trigo(5, pi/10)
(array([ 1.        ,  0.95105652,  0.80901699,  0.95105652,  1.        ,
        0.95105652]), array([ 0. , -0.30901699, -0.58778525, -0.30901699,  0. ,
       -0.30901699]))
"""


def moyenne_trigo(nb_pas, nb_marches, alpha):
    sx, sy = 0, 0
    for _ in range(nb_marches):
        x, y = marche_trigo(nb_pas, alpha)
        sx += x[-1]
        sy += y[-1]
    return sx/nb_marches, sy/nb_marches


def moyenne_theo(nb_pas, alpha):
    return cos(alpha)**nb_pas

"""
>>> moyenne_trigo(10, 10**5, pi/10)
(0.60663483107781901, 0.0036916962726706117)
>>> moyenne_theo(10, pi/10)
0.60542904971310629
"""

#END CUT


if __name__ == '__main__':
    print("dans le plan")
    # TODO
    # BEGIN CUT
    print(marche2(5))
    dessine_marche2()
    """
>>> marche2(5)
([0, 1, 0, 0, 0, 0], [0, 0, 0, 1, 2, 1])
>>> marche2(10)
([0, -1, -2, -3, -4, -3, -2, -1, 0, -1, -2], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
"""

"""
>>> distance_moyenne2(10, 10**4)
2.7972986400949882
>>> distance_carre_moyenne2(10, 10**4)
9.9152

>>> distance_moyenne2(100, 10**4)
8.8857170582871934
>>> distance_carre_moyenne2(100, 10**4)
99.4198
"""
"""
>>> proba_passage2((0, 0), 2, 10**4)
0.2543

>>> proba_passage2((0, 0), 10, 10**4)
0.4198
>>> proba_passage2((0, 0), 10, 10**6)
0.420706

>>> proba_passage2((0, 0), 100, 10**4)
0.5808
>>> proba_passage2((0, 0), 1000, 10**3)

0.662
>>> proba_passage2((-2, 0), 100, 10**3)
0.424
>>> proba_passage2((-2, 0), 1000, 10**3)
0.544

>>> proba_passage2((-5, 0), 100, 10**3)
0.147
>>> proba_passage2((-5, 0), 1000, 10**3)
0.349
"""

    # END CUT
